package org.eclipse.equinox.prov.shared.test;

import java.util.Iterator;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.NullProgressMonitor;
import org.eclipse.equinox.prov.director.IDirector;
import org.eclipse.equinox.prov.engine.Profile;
import org.eclipse.equinox.prov.metadata.IInstallableUnit;
import org.eclipse.equinox.prov.metadata.InstallableUnit;
import org.eclipse.equinox.prov.metadata.RequiredCapability;
import org.eclipse.equinox.prov.metadata.repository.IMetadataRepository;
import org.eclipse.equinox.prov.metadata.repository.IMetadataRepositoryManager;
import org.eclipse.equinox.prov.reconciler.IReconciler;
import org.eclipse.equinox.prov.reconciler.ProfileTemplate;
import org.eclipse.equinox.prov.reconciler.ReconcilerChanges;
import org.eclipse.equinox.prov.reconciler.SimpleReconciler;
import org.osgi.framework.Version;

public class UpdateReconciliation extends AbstractReconcilerTest
{
	public void testUpdate() throws CoreException
	{
		InstallableUnit iu1 = newIU("test.iu.a", "1.0.0");

		InstallableUnit iu2 = newIU("test.iu.b", "1.0.0");
		iu2.setRequiredCapabilities(new RequiredCapability[] {
				newRequired("test.iu.a", "1.0.0", false)
		});

		InstallableUnit iu3 = newIU("test.iu.c", "1.0.0");
		iu3.setRequiredCapabilities(new RequiredCapability[] {
				newRequired("test.iu.a", "1.0.0", false),
				newRequired("test.iu.b", "1.0.0", false)
		});

		InstallableUnit iu4 = newIU("test.iu.b", "2.0.0");
		iu4.setRequiredCapabilities(new RequiredCapability[] {
				newRequired("test.iu.a", "1.0.0", false)
		});

		InstallableUnit iu5 = newIU("test.iu.c", "2.0.0");
		iu5.setRequiredCapabilities(new RequiredCapability[] {
				newRequired("test.iu.a", "1.0.0", false),
				newRequired("test.iu.b", "2.0.0", false)
		});
		
		InstallableUnit iu6 = newIU("test.iu.d", "1.0.0");
		iu5.setRequiredCapabilities(new RequiredCapability[] {
				newRequired("test.iu.b", "[1.0.0,2.0.0)", false),
		});

		// Will contain:  a-1.0.0, b-1.0.0, c-1.0.0
		ProfileTemplate base = new ProfileTemplate(new Version("1.0.0"));
		base.setRequiredCapabilities(new RequiredCapability[] {
				newRequired("test.iu.a", "[1.0.0,1.0.0]", false),
				newRequired("test.iu.b", "[1.0.0,1.0.0]", false),
				newRequired("test.iu.c", "[1.0.0,1.0.0]", false)
		});

		// Will contain:  a-1.0.0, b-2.0.0, c-2.0.0
		ProfileTemplate baseUpgrade = new ProfileTemplate(new Version("2.0.0"));
		baseUpgrade.setRequiredCapabilities(new RequiredCapability[] {
				newRequired("test.iu.a", "[1.0.0,1.0.0]", false),
				newRequired("test.iu.b", "[2.0.0,2.0.0]", false),
				newRequired("test.iu.c", "[2.0.0,2.0.0]", false)
		});

		IInstallableUnit baseProfileIU = createIUFromTemplate(base);
		IInstallableUnit baseUpgradeProfileIU = createIUFromTemplate(baseUpgrade);
		
		// Set up metadata repository
		IMetadataRepositoryManager metadataMgr = getMetadataRepoMgr();
		IInstallableUnit ius[] = new IInstallableUnit[]{ iu1, iu2, iu3, iu4, iu5, iu6, 
				baseProfileIU, baseUpgradeProfileIU };

		IMetadataRepository metadataRepo = new TestMetadataRepository(ius);
		metadataMgr.addRepository(metadataRepo);
		try
		{
			IDirector director = getDirector();

			// First, let's create the "user" (for lack of a better term) profile
			Profile userProfile = newProfile("user");
			
			// Now, copy the base "shared" profile into the user profile
			IStatus status = director.install(new IInstallableUnit[] {baseProfileIU}, userProfile, "copy-of-shared", new NullProgressMonitor());
			assertOK(status);
			assertIUs(new IInstallableUnit[]{ iu1, iu2, iu3 }, userProfile.getInstallableUnits());
			printIUs(userProfile);
			
			// Now install into the user profile
			status = director.install(new IInstallableUnit[] {iu6}, userProfile, "user-installed" , new NullProgressMonitor());
			assertNotIUs(new IInstallableUnit[]{ iu4, iu5 }, userProfile.getInstallableUnits());
			printIUs(userProfile);
			
			// Now we'll do the reconciliation of an update, ensuring what's in the upgraded base profile is also in the user profile
			IReconciler reconciler = new SimpleReconciler();
			ReconcilerChanges rc = reconciler.reconcile(baseUpgradeProfileIU, userProfile, new NullProgressMonitor());
			assertTrue(rc.hadChanges());
			assertOK(rc.getStatus());
			printIUs(userProfile);
			
			assertIUs(new IInstallableUnit[]{ iu1, iu4, iu5, iu6 }, userProfile.getInstallableUnits());
			assertNotIUs(new IInstallableUnit[]{ iu2, iu3 }, userProfile.getInstallableUnits());

			deleteProfile(userProfile);
		}
		finally
		{
			metadataMgr.removeRepository(metadataRepo);
		}
	}

	private void printIUs(Profile profile) {
		System.out.println("IUs in " + profile.getProfileId() + ":");
		Iterator ius = profile.getInstallableUnits();
		while (ius.hasNext()) {
			IInstallableUnit iu = (IInstallableUnit) ius.next();
			String entryPoint = iu.getProperty("entryPoint");
			if (entryPoint != null && entryPoint.equalsIgnoreCase("true"))
				System.out.println("\t" + iu + " (entryPoint)");
			else
				System.out.println("\t" + iu);
		}
	}
//
//	private void printUninstalledSoftware(ReconcilerChanges rc) {
//		System.out.println("Uninstalled stuff:");
//		IResolvedInstallableUnit[] uninstalledIUs = rc.getUninstalledSoftware();
//		for (int i = 0; i < uninstalledIUs.length; i++) {
//			System.out.println(uninstalledIUs[i]);
//		}
//	}
//
//	private void printUpdatedSoftware(ReconcilerChanges rc) {
//		System.out.println("Updated stuff:");
//		IResolvedInstallableUnit[] updatedIUs = rc.getUpdatedSoftware();
//		for (int i = 0; i < updatedIUs.length; i++) {
//			System.out.println(updatedIUs[i]);
//		}
//	}
//
//	private void printInstalledSoftware(ReconcilerChanges rc) {
//		System.out.println("Installed stuff:");
//		IResolvedInstallableUnit[] installedIUs = rc.getInstalledSoftware();
//		for (int i = 0; i < installedIUs.length; i++) {
//			System.out.println(installedIUs[i]);
//		}
//	}
//
//	private void printUninstallOperands(ReconcilerChanges rc) {
//		List uninstallOps = rc.getUninstallOperands();
//		if (uninstallOps != null && uninstallOps.size() > 0)
//			System.out.println("Uninstall operands:");
//		for (Iterator iterator = uninstallOps.iterator(); iterator.hasNext();) {
//			UninstallOperand uninstallOp = (UninstallOperand) iterator.next();
//			System.out.println(uninstallOp);
//		}
//	}
//
//	private void printInstallOperands(ReconcilerChanges rc) {
//		List installOps = rc.getInstallOperands();
//		if (installOps != null && installOps.size() > 0)
//			System.out.println("Install operations:");
//		for (Iterator iterator = installOps.iterator(); iterator.hasNext();) {
//			InstallOperand installOp = (InstallOperand) iterator.next();
//			System.out.println(installOp);
//		}
//	}
//	
//	private void printUpdateOperands(ReconcilerChanges rc) {
//		List updateOps = rc.getUpdateOperands();
//		if (updateOps != null && updateOps.size() > 0)
//			System.out.println("Install operations:");
//		for (Iterator iterator = updateOps.iterator(); iterator.hasNext();) {
//			UpdateOperand installOp = (UpdateOperand) iterator.next();
//			System.out.println(installOp);
//		}
//	}
}